#include "XiaoFS.hpp"

XiaoFS::XiaoFS() {}

void XiaoFS::init() {
  while (!Serial) continue;

  if (!myFS->init()) {
    Serial.println("FS Mount Failed");
    return;
  }

  Serial.print("\nStart FS_Test on "); Serial.println(BOARD_NAME);
  Serial.println(FS_NANO33BLE_VERSION);

  #if defined(FS_NANO33BLE_VERSION_MIN)
    if (FS_NANO33BLE_VERSION_INT < FS_NANO33BLE_VERSION_MIN) {
      Serial.print("Warning. Must use this example on Version equal or later than : ");
      Serial.println(FS_NANO33BLE_VERSION_MIN_TARGET);
    }
  #endif

  Serial.print("FS_size (KB) = "); Serial.println(NANO33BLE_FS_SIZE_KB);
  Serial.print("FS_ Start Address = 0x"); Serial.println(NANO33BLE_FS_START, HEX);
}

void XiaoFS::readCharsFromFile(const char * path) {
  Serial.print("readCharsFromFile: "); Serial.print(path);

  FILE *file = fopen(path, "r");
  
  if (file) 
  {
    Serial.println(" => Open OK");
  }
  else
  {
    Serial.println(" => Open Failed");
    return;
  }

  char c;

  while (true) 
  {
    c = fgetc(file);
    
    if ( feof(file) ) 
    { 
      break;
    }
    else   
      Serial.print(c);
  }
   
  fclose(file);
}

bool XiaoFS::readFile(const char * path) {
  Serial.print("Reading file: "); Serial.print(path);

  // Open file reader
  FILE *file = fopen(path, "r");
  if (file) {
    Serial.println(" => Open OK");
  } else {
    Serial.println(" => Open Failed");
    return false;
  }

  // Read the file
  char c;
  uint32_t numRead = 1;
  std::string json = "";
  while (numRead) {
    numRead = fread((uint8_t *) &c, sizeof(c), 1, file);

    if (numRead) {
      //Serial.print(c);
      json+=c;
    }
  }
  //Serial.println(json.c_str());

  // Convert it back to a Json OBJECT
  StaticJsonDocument<200> doc;
  DeserializationError error = deserializeJson(doc, json);
  if (error) {
    Serial.print(F("deserializeJson() failed: "));
    Serial.println(error.f_str());
    
    fclose(file);
    return false;
  }
  config_file.MACs.clear();
  JsonArray jsonMACs = doc["MACs"].as<JsonArray>();
  for(JsonVariant mac : jsonMACs) {
      config_file.MACs.push_back(mac);
  }
  
  fclose(file);
  return true;
}

//void XiaoFS::writeFile(const char * path, const char * message, size_t messageSize) {
bool XiaoFS::writeFile(const char * path) {
  Serial.print("Writing file: "); Serial.print(path);

  // Open filw writer
  FILE *file = fopen(path, "w");
  if (file) {
    Serial.println(" => Open OK");
  } else {
    Serial.println(" => Open Failed");
    return false;
  }

  // Transform to a JSON string
  std::string tmp;
  StaticJsonDocument<200> doc;
  JsonArray data = doc.createNestedArray("MACs");
  for (const auto& mac : config_file.MACs) {
    data.add(mac);
  }
  serializeJson(doc, tmp);
  //Serial.println(tmp);
  

  // Write to file
  if (fwrite(tmp.c_str(), sizeof(char), tmp.length(), file)) {
    Serial.println("* Writing OK");
  } else {
    Serial.println("* Writing failed");
    
    fclose(file);
    return false;
  }
  
  fclose(file);
  return true;
}

void XiaoFS::appendFile(const char * path, const char * message, size_t messageSize) {
  Serial.print("Appending file: "); Serial.print(path);

  FILE *file = fopen(path, "a");
  
  if (file) 
  {
    Serial.println(" => Open OK");
  }
  else
  {
    Serial.println(" => Open Failed");
    return;
  }

  if (fwrite((uint8_t *) message, 1, messageSize, file)) 
  {
    Serial.println("* Appending OK");
  } 
  else 
  {
    Serial.println("* Appending failed");
  }
   
  fclose(file);
}

bool XiaoFS::deleteFile(const char * path) {
  Serial.print("Deleting file: "); Serial.print(path);
  // Sync struct
  config_file.MACs.clear();
  
  if (remove(path) == 0) {
    Serial.println(" => OK");
    return true;
  } else {
    Serial.println(" => Failed");
    return false;
  }
}

void XiaoFS::renameFile(const char * path1, const char * path2) {
  Serial.print("Renaming file: "); Serial.print(path1);
  Serial.print(" to: "); Serial.print(path2);
  
  if (rename(path1, path2) == 0) 
  {
    Serial.println(" => OK");
  }
  else
  {
    Serial.println(" => Failed");
    return;
  }
}

void XiaoFS::testFileIO(const char * path) {
  Serial.print("Testing file I/O with: "); Serial.print(path);

  #define BUFF_SIZE     512
  
  static uint8_t buf[BUFF_SIZE];
  
  FILE *file = fopen(path, "w");
  
  if (file) 
  {
    Serial.println(" => Open OK");
  }
  else
  {
    Serial.println(" => Open Failed");
    return;
  }

  size_t i;
  Serial.println("- writing" );
  
  uint32_t start = millis();

  size_t result = 0;

  // Write a file only 1/4 of NANO33BLE_FS_SIZE_KB
  for (i = 0; i < NANO33BLE_FS_SIZE_KB / 2; i++) 
  {
    result = fwrite(buf, BUFF_SIZE, 1, file);

    if ( result != 1)
    {
      Serial.print("Write result = "); Serial.println(result);
      Serial.print("Write error, i = "); Serial.println(i);

      break;
    }
  }
  
  Serial.println("");
  uint32_t end = millis() - start;
  
  Serial.print(i / 2);
  Serial.print(" Kbytes written in (ms) ");
  Serial.println(end);
  
  fclose(file);

  printLine();

  /////////////////////////////////

  file = fopen(path, "r");
  
  start = millis();
  end = start;
  i = 0;
  
  if (file) 
  {
    start = millis();
    Serial.println("- reading" );

    result = 0;

    fseek(file, 0, SEEK_SET);

    // Read file only 1/4 of NANO33BLE_FS_SIZE_KB
    for (i = 0; i < NANO33BLE_FS_SIZE_KB / 2; i++) 
    {
      result = fread(buf, BUFF_SIZE, 1, file);

      if ( result != 1 )
      {
        Serial.print("Read result = "); Serial.println(result);
        Serial.print("Read error, i = "); Serial.println(i);

        break;
      }
    }
      
    Serial.println("");
    end = millis() - start;
    
    Serial.print((i * BUFF_SIZE) / 1024);
    Serial.print(" Kbytes read in (ms) ");
    Serial.println(end);
    
    fclose(file);
  } 
  else 
  {
    Serial.println("- failed to open file for reading");
  }
}

void XiaoFS::printLine() {
  Serial.println("\n****************************************************");
}

void XiaoFS::printData() {
  Serial.println("MACs:");
  for (const auto& mac : config_file.MACs) {
    Serial.print("\t"); Serial.println(mac.c_str());
  }
}
